class Statment(object):
    def __init__(self):
        self.account_name = ""
        self.account_number = ""
        self.iban = ""
        self.currency = ""
        self.number = -1
        self.from_date = None
        self.to_date = None
        self.opening_balance = 0
        self.income = 0
        self.expenses = 0
        self.closing_balance = 0
        self.blocked = 0
        self.receivable = 0
        self.available_balance = 0
        self.movements = []
    



import re
from .utils import (
    to_decimal,
    money_regex
)

# parse the account summary - which will look something like this:

"""
Beginning balance                                                                 0.00
Income                                               1 000.00                 1 000.00
Expense                                                  0.00                     0.00
Ending balance                                                                1 000.00
Of which, blocked                                                                 0.00
Receivables past due                                                              0.00
Available balance                                                             1 000.00
"""

# pretty heavy on the regexes - maybe need to think about using ply (python lex/yacc)

opening_regex = r"Beginning balance\s+(%s)" % (money_regex)
income_regex = r"Income\s+(%s)\s+(%s)" % (money_regex, money_regex)
expenses_regex = r"Expense\s+(%s)\s+(%s)" % (money_regex, money_regex)
closing_regex = r"Ending balance\s+(%s)" % (money_regex)
blocked_regex = r"Of which, blocked\s+(%s)" % (money_regex)
receivable_regex = r"Receivables past due\s+(%s)" % (money_regex)
available_regex = r"Available balance\s+(%s)" % (money_regex)

from pprint import pprint

class AccountSummaryParser(object):

    def Parse(self, statement, section):

        parsers = [
            self.parse_opening,
            self.parse_income,
            self.parse_expenses,
            self.parse_closing,
            self.parse_blocked,
            self.parse_due
        ]

        for line in section:
            #print "\"" + line + "\""
            
            if not line:
                continue

            for parser in parsers:
                if parser(statement, line):
                    break
        
    def parse_opening(self, statement, line):
        parsed_opening = re.match(opening_regex, line)
        if parsed_opening:
            statement.opening_balance = to_decimal(parsed_opening.group(1))
            return True
        return False

    def parse_income(self, statement, line):
        parsed_income = re.match(income_regex, line)
        if parsed_income:
            #statement.ytd_income = to_decimal(parsed_income.group(1))
            statement.income = to_decimal(parsed_income.group(2))
            return True
        return False

    def parse_expenses(self, statement, line):
        parsed_expenses = re.match(expenses_regex, line)
        if parsed_expenses:
            #statement.ytd_expenses = to_decimal(parsed_expenses.group(1))
            statement.expenses = to_decimal(parsed_expenses.group(2))
            return True
        return False
    
    def parse_closing(self, statement, line):
        parsed_closing = re.match(closing_regex, line)
        if parsed_closing:
            statement.closing_balance = to_decimal(parsed_closing.group(1))
            return True
        return False

    def parse_blocked(self, statement, line):
        parsed_blocked = re.match(blocked_regex, line)
        if parsed_blocked:
            statement.blocked = to_decimal(parsed_blocked.group(1))
            return True
        return False

    def parse_due(self, statement, line):
        parsed_receivable = re.match(receivable_regex, line)
        if parsed_receivable:
            statement.receivable = to_decimal(parsed_receivable.group(1))
            return True
        return False

    def parse_available(self, statement, line):
        parsed_available = re.match(available_regex, line)
        if parsed_available:
            statement.available_balance = to_decimal(parsed_available.group(1))
            return True
        return False



import re

from .statement_header_parser import StatementHeaderParser
from .account_summary_parser import AccountSummaryParser
from .account_movements_parser import AccountMovementsParser

from .statement import *

section_regex = "^=+$"

class StatementParser(object):

    def __init__(self):
        self.parsers = [
            StatementHeaderParser(),
            AccountSummaryParser(),
            AccountMovementsParser()
        ]

    def split_into_sections(self, file_contents):

        sections = []
        current_section = []
        
        for raw_line in file_contents:
            line = raw_line.strip()
            
            if re.match(section_regex, line):
                if len(current_section) > 3 and not self.skip_section(current_section):
                    sections.append(current_section)
                current_section = []
                continue

            current_section.append(line)

        if len(current_section) > 3 and not self.skip_section(current_section):
            sections.append(current_section)
            
        return sections

    def skip_section(self, section):
        if (len(section) <= 1):
            return True

        if section[0] == "Message for client":
            return True

        return False
    
    def Parse(self, file_contents):

        parser_idx = 0
        sections = self.split_into_sections(file_contents)
        statement = Statment()
        
        for section in sections:
            for parser in self.parsers[parser_idx:]:
                parser.Parse(statement, section)
                parser_idx += 1
                break
                
        return statement
                



import re
from datetime import datetime
from .movement import Movement
from .utils import (
    to_decimal,
    money_regex
)

# meant to parse a section like the below - lines of transactions split up
# by series of dashes

"""
   1 02.08.RB Ceska, Brno, CZE   29.07.                            -1 800.00          
     10:57 Debit Card:516872XXXXX                                                     
           8323453/5500                     1178       Withdraw from ATM              
--------------------------------------------------------------------------------------
   2 02.08.RB Ceska, Brno, CZE   31.07.                            -1 800.00          
     10:57 Debit Card:516872XXXXX                                                     
           8323453/5500                     1178       Withdraw from ATM              
--------------------------------------------------------------------------------------
   3 02.08.                      31.07.                              -617.20          
     10:57 Debit Card:516872XXXXX                                                     
           8323525/5500                     1178       Card payment                   
           Billa Namesti Svobody, Brno - Omega, CZE
--------------------------------------------------------------------------------------
"""

delimiter_regex = "^-+$"

class AccountMovementsParser(object):

    def Parse(self, statement, section):
        movements = self.split_into_movements(section)

        for movement in movements:
            if len(movement) > 1:
                m = Movement()
                #for line in movement:
                self.parse_first_line(m, movement[0])
                statement.movements.append(m)
                
    def split_into_movements(self, section_contents):
        movements = []
        current_movement = []

        for line in section_contents:
            if re.match(delimiter_regex, line):
                movements.append(current_movement)
                current_movement = []
                continue

            current_movement.append(line)

        return movements       
    
    def parse_first_line(self, movement, line):
        first_regex = r"\s*(\d+)\s+(\d\d\.\d\d)\.(.*)(\d\d\.\d\d)\.\s{5}\d?\s+(%s)" % (money_regex)

        first_match = re.match(first_regex, line)

        if first_match:
            movement.number = first_match.group(1)
            movement.date = first_match.group(4)
            movement.amount = to_decimal(first_match.group(5))
            



from .statement_parser import StatementParser

def read_statement(filename):
    with open(filename) as f:
        return StatementParser().Parse(f.readlines())

def read_statements(filenames):
    return [ read_statement(filename) for filename in filenames ]

def read_statements_from_mailbox(imap_addr, username, password, mailbox="inbox"):
    """
    from https://yuji.wordpress.com/2011/06/22/python-imaplib-imap-example-with-gmail/
    something like...

    import imaplib
    
    M = imaplib.IMAP4()
    M.login(getpass.getuser(), getpass.getpass())
    M.select()
    typ, data = M.search(None, 'ALL')
    for num in data[0].split():
        typ, data = M.fetch(num, '(RFC822)')
        print 'Message %s\n%s\n' % (num, data[0][1])
    M.close()
    M.logout()

    """
    pass
    
    



import re
from datetime import datetime

# what we're needing to do is parse a chunk of text like the following

"""
Raiffeisenbank a.s. 
Bank statement No. 8
For period 01.08.2016/31.08.2016

Name of account: Sean James McLemon
Account number: 6986230001/5500
IBAN:            CZ3555000000006986230001
Currency:        CZK
"""

# how we'll do this is
# 1. strip blank lines (there's a few)
# 2. retrieve the bank statement number
# 3. retrieve the from/to dates
# 4. pick out the values of the form "name: value" and strip() them

statement_number_regex = "Bank statement No. (\d+)"
date_regex = "\d\d\.\d\d\.\d\d\d\d"
for_period_regex = "For period (%s)/(%s)" % (date_regex, date_regex)
colon_delimit_regex = "(.*)\:(.*)"

account_name_label = "Name of account"
account_number_label = "Account number"
iban_label = "IBAN"
currency_label = "Currency"

class StatementHeaderParser(object):

    def Parse(self, statement, section):

        parsers = [
            self.parse_statement_number,
            self.parse_from_to,
            self.parse_assign
        ]

        for line in section:
            if not line:
                continue

            for parser in parsers:
                if parser(statement, line):
                    break

    def parse_statement_number(self, statement, line):
        parsed_stmt_number = re.match(statement_number_regex, line)
        
        if parsed_stmt_number:
            statement.number = parsed_stmt_number.group(1)
            return True
        return False

    def parse_from_to(self, statement, line):
        parsed_dates = re.match(for_period_regex, line)
        if parsed_dates:
            statement.from_date = datetime.strptime(parsed_dates.group(1), "%d.%m.%Y")
            statement.to_date = datetime.strptime(parsed_dates.group(2), "%d.%m.%Y")            
            return True
        return False
            
    def parse_assign(self, statement, line):
        
        parsed_assign = re.match(colon_delimit_regex, line)
        if parsed_assign:
            label = parsed_assign.group(1).strip()
            value = parsed_assign.group(2).strip()

            if (label == account_name_label):
                statement.account_name = value
            elif (label == account_number_label):
                statement.account_number = value
            elif (label == iban_label):
                statement.iban = value
            elif (label == currency_label):
                statement.currency = value
            else:
                return False
            return True
        return False
    



from distutils.core import setup
setup(
  name = 'rbcz',
  packages = ['rbcz'],
  version = '0.1',
  description = 'library for interacting with Czech Raiffeisen Bank\'s text bank statements',
  author = 'Sean McLemon',
  author_email = 'sean.mclemon@gmail.com',
  url = 'https://github.com/smcl/rbcz.py',
  download_url = 'https://github.com/smcl/rbcz.py/tarball/0.1',
  keywords = ['banking', 'raiffeisen', 'czech', 'cz'],
  classifiers = [],
)
